<?php

namespace Form\FullForm;

use Form\Constraint\ConstraintInterface;
use InvalidArgumentException;

trait FieldTrait
{
    /**
     * {@inheritdoc}
     */
    public function isValid()
    {
        $value = $this->getValue();
        return array_reduce(
            $this->getConstraints(),
            function ($isValid, ConstraintInterface $constraint) use ($value) {
                return $isValid && $constraint->isValid($value);
            },
            true
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getOptionByValue($value)
    {
        return array_reduce(
            $this->getOptions(),
            function (OptionInterface $foundOption = null, OptionInterface $option) use ($value) {
                if ($foundOption !== null) {
                    return $foundOption;
                }

                if ($option->getValue() === $value) {
                    return $option;
                }

                return null;
            }
        );
    }

    /**
     * {@inheritdoc}
     */
    public function getSelectedValue()
    {
        return $this->getOptionByValue($this->getValue());
    }

    /**
     * @param $tag
     * @return bool
     */
    public function hasTag($tag)
    {
        if (empty($tag)) {
            throw new InvalidArgumentException("Non empty tag expected");
        }

        return in_array($tag, $this->getTags(), true);
    }

    /**
     * @return OptionInterface[]
     */
    abstract public function getOptions();

    /**
     * @return mixed
     */
    abstract protected function getValue();

    /**
     * @return ConstraintInterface[]
     */
    abstract protected function getConstraints();

    /**
     * @return string[]
     */
    abstract protected function getTags();
}
