<?php

namespace Form\FullForm;

use InvalidArgumentException;

trait FormTrait
{
    /**
     * @return bool
     */
    public function isValid()
    {
        return array_reduce($this->getFields(), function ($isValid, FieldInterface $field) {
            return $isValid && $field->isValid();
        }, true);
    }

    /**
     * @param string $name
     * @return FieldInterface|null
     */
    public function getField($name)
    {
        return array_reduce(
            $this->getFields(),
            function (FieldInterface $found = null, FieldInterface $field) use ($name) {
                if ($found !== null) {
                    return $found;
                }

                return $field->getName() === $name ? $field : null;
            }
        );
    }


    /**
     * @param string $tag
     * @return FieldInterface[]
     */
    public function getFieldsByTag($tag)
    {
        if (empty($tag)) {
            throw new InvalidArgumentException("Non empty tag expected");
        }

        return array_filter(
            $this->getFields(),
            function (FieldInterface $field) use ($tag) {
                return $field->hasTag($tag);
            }
        );
    }

    /**
     * {@inheritdoc}
     */
    abstract public function getFields();
}
