<?php

namespace Form\FullForm;

/**
 * Class Option
 *
 * Simple implementation of OptionInterface
 */
class Option implements OptionInterface
{
    /**
     * @var string
     */
    private $value;

    /**
     * @var string
     */
    private $name;

    /**
     * @var bool
     */
    private $selected;

    /**
     * @var bool
     */
    private $disabled;

    /**
     * Option constructor.
     * @param string $value
     * @param string $name
     * @param bool $selected
     * @param bool $disabled
     */
    public function __construct($value, $name, $selected, $disabled)
    {
        $this->value = $value;
        $this->name = $name;
        $this->selected = $selected;
        $this->disabled = $disabled;
    }

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @return bool
     */
    public function isSelected()
    {
        return $this->selected;
    }

    /**
     * @return bool
     */
    public function isDisabled()
    {
        return $this->disabled;
    }

    /**
     * @param string $value
     * @return OptionInterface
     */
    public function withValue($value)
    {
        return new self(
            $value,
            $this->getName(),
            $this->isSelected(),
            $this->isDisabled()
        );
    }

    /**
     * @param string $name
     * @return OptionInterface
     */
    public function withName($name)
    {
        return new self(
            $this->getValue(),
            $name,
            $this->isSelected(),
            $this->isDisabled()
        );
    }

    /**
     * @param bool $selected
     * @return OptionInterface
     */
    public function withSelected($selected)
    {
        return new self(
            $this->getValue(),
            $this->getName(),
            $selected,
            $this->isDisabled()
        );
    }

    /**
     * @param bool $disabled
     * @return OptionInterface
     */
    public function withDisabled($disabled)
    {
        return new self(
            $this->getValue(),
            $this->getName(),
            $this->isSelected(),
            $disabled
        );
    }
}
